; *****************************************************************************
; *****************************************************************************
;
;       File    : Data.Asm
;       Author  : Paul Robson (autismuk@aol.com)
;       Purpose : Data Segment / Reset Code
;       Created : 24th September 1997
;       Modified: 12th November 1997
;
;       Functions :
;               DReset          Reset the Data Area, CPU etc.
;
; *****************************************************************************
; *****************************************************************************


.386p 
.model flat
include extern.inc
.code

; *****************************************************************************
;
;                               Reset the system
;
; *****************************************************************************

DReset Proc
        push    eax
        push    ebx
        push    edx

        mov     eax,[FrmRate]           ; save frame rate
        push    eax                     ; reset doesn't change this.

        xor     eax,eax                 ; Clear the main area to all zero
        mov     ebx,offset DStartData
_DRS1:  mov     [ebx],eax
        add     ebx,4
        cmp     ebx,offset DEndData
        jb      _DRS1

        mov     [_SP],0F000h            ; Set SP = $F000, apparently default !
        mov     [_IFF1],1               ; Turn on Global Interrupt Enable
        mov     [_IFF2],1
        mov     [FrameCount],1          ; Frame DownCounter

        cmp     [IsAsmRom],0            ; test ROM tyupe AszMic ?
        jnz     _DRS4
        cmp     [IsZX80Rom],0           ; test ROM type ZX80 ?
        jnz     _DRS2
        call    ROMZX81                 ; load the 81 ROM
        jmp     _DRS3
_DRS4:  call    ROMAszMic               ; load the AszMic Rom
        jmp     _DRS3
_DRS2:  call    ROMZX80                 ; load the 80 ROM
_DRS3:

;
;       for 16 bit I reads on $FFFF e.g. LD HL,($FFFF) could be some wierd
;       Int trick, the Spectrum needs it. Copy first byte of ROM to address
;       past $FFFF so 16 bit reads work properly - because I do them
;       in a single read rather than 8 byte chunks.
;

        mov     al,[byte ptr Memory+0]  
        mov     [byte ptr Memory+10000h],al

        mov     [_RBit7],128            ; initialise R bit 7 to 1
                                        ; Alien-8 didn't work without this

        pop     eax                     ; restore framerate
        mov     [FrmRate],eax           ; (not affected by DReset)
        call    SCResetBackground       ; reset the colour scheme

        pop     edx
        pop     ebx
        pop     eax
        ret

EndP DReset

; *****************************************************************************
; *****************************************************************************
;
;                               System Data Area
;
;       This starts with some variables which aren't in the
;       Snapshot, followed by the 64k memory block.
;
; *****************************************************************************
; *****************************************************************************

.data?
align   dword

DStartData:

Memory: db      04000h dup (?)          ; 16k of Memory (for ROM Image)
        db      0C100h dup (?)          ; remaining 48k + 1/4k spare

ExecPtr dd      ?                       ; Execution address pointer
DbgData dw      ?,?                     ; Debug Data Pointer
Break   dw      ?,?                     ; Debugger Breakpoint
FrmRate dd      ?                       ; Frames per PC Clock Tick
LastSync dw     ?,?                     ; Last timer value
Cycles  dd      ?                       ; Cycle Count
FrameCount dd   ?                       ; Frame Counter
VSyncCount dd   ?                       ; Vertical Sync Counter
NMIEnabled dd   ?                       ; NMI Enabled Flag
NMICounter dd   ?                       ; counts the number of NMIs
NMIVSCounter dd ?                       ; since last VSYNC
VideoLine dd    ?                       ; Video Line Counter
ULALine dd      ?                       ; ULA Line Counter
ULADelay dd     ?                       ; ULA Delay Counter
Scheme  dd      ?                       ; Colour Scheme
Background dd   ?                       ; Background Colour
Foreground dd   ?                       ; Foreground Colour
Redraw  dd      ?                       ; Redraw flag
     
_AF:                                    ; Z80 Main Register Set
_F      db      ?                       ; 4 main register pairs
_A      db      ?,?,?
_BC:
_C      db      ?
_B      db      ?,?,?
_DE:
_E      db      ?
_D      db      ?,?,?
_HL:
_L      db      ?
_H      db      ?,?,?

_IX     dw      ?,?                     ; Index registers
_IY     dw      ?,?
_SP     dw      ?,?                     ; Stack pointer
_PC     dw      ?,?                     ; Program Counter

_XAF    dw      ?,?                     ; Alternate Register Set
_XBC    dw      ?,?
_XDE    dw      ?,?
_XHL    dw      ?,?

_IMode  db      ?,?,?,?                 ; Interrupt Mode (0,1,2)
_IFF1   db      ?,?,?,?                 ; Interrupt enable flags
_IFF2   db      ?,?,?,?
_I      db      ?,?,?,?                 ; I Register
_R      db      ?,?,?,?                 ; R Register : bit 7 isn't masked.
                                        ; until LD A,R.
_RBit7  db      ?,?,?,?


; *****************************************************************************

DEndData:

end
