; *****************************************************************************
; *****************************************************************************
;
;       File    : Main.Asm
;       Author  : Paul Robson (autismuk@aol.com)
;       Purpose : Main Program, ZX81 Emulator
;       Created : 12th September 1997
;       Modified: 12th November 1997
;
; *****************************************************************************
; *****************************************************************************

.386p 
.model flat
include extern.inc

.code 

Start:  ; jmp     KTest                 ; remove ; to check scancodes
        mov     [FrmRate],9             ; boot in turbo mode
        call    CMHandler               ; Command Line Handler
        call    KInitialise             ; Initialise new keyboard handler
        call    VSetGraphicMode         ; Switch to 300x256 mode

Reset1: call    DReset                  ; Reset the CPU
        call    IScanKeys               ; Scan the keys initially
        mov     al,dbmRun               ; switch to run normal mode
        call    BSetExecute

Main1:                                  ; *** MAIN LOOP ***
    
NMICycle:
        call    [ExecPtr]               ; Execute a single instruction
        cmp     [Cycles],208            ; done 208 cycles ? (64us @ 3.25 Mhz)
        jl      NMICycle                ; if not loop back

        sub     [Cycles],208            ; reset TState Counter back

        cmp     [NMIEnabled],0          ; check NMI is enabled
        jz      NoNMI                   ; (switched by OUT to FD,FE)
        cmp     [IsZX80Hardware],0      ; check not ZX80 hardware
        jnz     NoNMI                   ; which has no NMI generator.

        cmp     [IsRealNMI],0           ; Always execute the NMI Code
        jnz     NoQuickNMI
        cmp     [byte ptr _XAF+1],255   ; quick NMI fix to speed emu up
        jne     QuickNMI                ; comment this line to do NMI properly.

NoQuickNMI:
        call    _OG2_FE                 ; generate an NMI
        jmp     NoNMI

QuickNMI:call   HaltCheck               ; short cut the NMI call.
        inc     [byte ptr _XAF+1]       ; if 'A is not 255 it's just an INC 'A
        add     [Cycles],58             ; extra cycles used

NoNMI:  inc     [NMICounter]            ; check keys every 284 Scan Lines
        inc     [NMIVSCounter]          ; counter for vertical blank

        mov     eax,[NMICounter]        ; test for 284 scan lines counter
        cmp     eax,284
        jb      NMICycle
        mov     [NMICounter],0          ; reset for next time

        call    IScanKeys               ; Rescan the keys
        dec     [FrameCount]            ; Decrement Sync counter

_ReSync:mov     eax,[FrameCount]        ; speed sync required ?
        or      eax,eax                 ; if counter > 0
        jg      NoSyncReqd              ; we don't need it yet.

        cmp     [VSyncCount],0          ; if this is zero , fast mode ?
        jnz     _NoFast

        call    SCFastMode              ; special fast mode display for the
                                        ; sad

_NoFast:mov     [VSyncCount],0          ; reset the counter

        mov     eax,[FrmRate]           ; reset frame count
        add     [FrameCount],eax        ; add to counter
        cmp     eax,9                   ; speed 9 has no timer wait
        je      NoSyncReqd

_WaitTm:call    ReadPCTimer             ; wait for timer to change
        cmp     ax,[LastSync]
        je      _WaitTM
        mov     [LastSync],ax
        jmp     _ReSync

NoSyncReqd:                             ; Test for various keys
        test    [byte ptr KeyTable+kF10],128
        jnz     GoDebugger              ; F10 Enters the Debugger
        test    [byte ptr KeyTable+kF1],128
        jnz     Reset1                  ; F1 is Reset
        test    [byte ptr KeyTable+kF2],128
        jnz     SchemeChg               ; F2 is Scheme
        test    [byte ptr KeyTable+kXPlus],128
        jnz     SpeedToggle             ; Keypad + is Speed Toggle

        test    [byte ptr KeyTable+kEsc],128
        jz      Main1                   ; Loop back if ESC not pressed

        jmp     GoDOS                   ; return to MSDOS

BError:
        call    BDebug                  ; Come here on illegal opcode
        jmp     GoDos                   ; exit to DOS

SchemeChg:                              ; Change Colour Scheme
        test    [byte ptr KeyTable+kF2],128
        jnz     SchemeChg
        inc     [Scheme]                ; next scheme
        and     [Scheme],3              ; of four
        call    SCResetBackground       ; reset the background
        jmp     Main1

GoDebugger:                             ; Enter Debugger
        mov     al,dbmSingleStep        ; different execution routine
        call    BSetExecute
        jmp     Main1

SpeedToggle:                            ; Toggle Speed
        test    [byte ptr KeyTable+kXPlus],128
        jnz     SpeedToggle
        cmp     [FrmRate],9
        je      _ST1
        mov     [FrmRate],9
        call    SCFrame
        jmp     Main1
_ST1:   mov     [FrmRate],FrameStd
        call    SCFrame
        jmp     Main1
     
GoDOS:  call    VSetTextMode            ; Time for tubby bye-byes
        call    KTerminate              ; patch video, keyboard
        mov     ah,9                    ; tell you about me
        mov     edx,offset Terminate    ; if you really want to know
        int     21h
        mov     ax,04c00h               ; return to MSDOS
        int     21h

Terminate:
        db      'ZX81 Emulator : '
        include build.inc
        db      13,10,'Written by Paul Robson (C) 1997.',13,10
        db      'Mail to autismuk@aol.com.',13,10
        db      'Written using Michael Tippich''s Dos Extender WDOSX 0.94.',13,10
        db      'and Borland Turbo C 2.0 and Turbo Assembler 4.0.',13,10
        db      'This program is freeware. ',13,10
        db      10,'$'
QuitMenu:db     2,'Return to Emulator',0,'Exit to DOS',0
        align   dword

; ***********************************************************************
;
;                      Read the PC's 18.2 Hz Timer
;
; ***********************************************************************

ReadPCTimer Proc
        push    ebx ecx edx
        mov     ah,0                    ; look at the clock tick
        int     01Ah
        mov     ax,dx                   ; put the tick in AX
        pop     edx ecx ebx
        ret
EndP ReadPCTimer


; *****************************************************************************
;
;                     Check if on a HALT, and bump PC if so.
;
; *****************************************************************************
 
HaltCheck Proc
        mov     ebx,[dword ptr _PC]     ; if next command is $76 (Halt)
        and     ebx,07FFFh              ; mask for Display File !
        mov     al,[byte ptr Memory+ebx]
        cmp     al,076h
        jne     _CI1
        inc     [word ptr _PC]          ; bump PC (wake it up)
_CI1:   ret
EndP HaltCheck
    
; *****************************************************************************
;
;                   Check Interrupt & Execute if Enabled
;                     (Generates a Maskable Interrupt)
;
; *****************************************************************************

CheckInterrupt Proc
        mov     al,[_Iff1]              ; this is an IRQ - check to see
        or      al,al                   ; if it is enabled
        jz      _CI0

        mov     [_IFF2],0               ; IFF2 = 0
        mov     [_IFF1],0               ; IFF1 = 0

        call    HaltCheck               ; bump PC if on HALT

        cmp     [_IMode],2              ; IM2 ?
        je      _CI2
        call    _OG0_FF                 ; IM1 & 0 both do RST38h
        jmp     _CI0

_CI2:   call    _OG2_FF                 ; IM2 does CALL (I)FFh

_CI0:   ret
EndP CheckInterrupt

; *****************************************************************************
;
;                             Load Tape Handler
;
;       Patched in at 0343 after the filename is patched. Loads code into
;       Z80 Map at 4009 and jumps to the RST08h with code 255 (ok) 13 (fail)
;
; *****************************************************************************

LoadTapePatch Proc
        mov     [word ptr _PC],03A6h    ; jump to RST08 call afterwards
        mov     [byte ptr Memory+03A7h],255

        mov     ax,0FEFEh               ; port activity
        call    IInPort                 ; makes compatibility better
        mov     al,0FFh
        call    IOutPort

        call    ExtractFileName         ; get the filename

        mov     eax,EXT_P               ; load a .P file
        call    CMFileName              ; get that file name
        mov     edx,ebx                 ; name in EDX
        mov     ebx,offset Memory+4009h ; load program address in EBX
        call    FLoad
        or      eax,eax                 ; if loaded,change exit
        jnz     _LTP0

        mov     eax,EXT_81              ; load a .81 file instead ?
        call    CMFileName              ; get that file name
        mov     edx,ebx                 ; name in EDX
        mov     ebx,offset Memory+4009h ; load program address in EBX
        call    FLoad
        or      eax,eax                 ; if loaded,change exit
        jnz     _LTP0
                                        ; error code 13 will be returned
        mov     [byte ptr Memory+03A7h],12
_LTP0:
        ret
EndP LoadTapePatch

; *****************************************************************************
;
;         DE points to a ZX81 Format String, Copy to Filename
;
; *****************************************************************************

ExtractFileName Proc
        mov     edx,[dword ptr _DE]     ; DE points to the filename
        add     edx,offset Memory       ; edx points to real memory !
        mov     al,[edx]                ; file name update first character
        cmp     al,0Bh                  ; if (HL)=$0B then it was LOAD""
        je      _LTP1                   ; so go to the loading routine

        mov     ebx,offset FNTemp       ; copy the filename in the buffer
        mov     [FileName],ebx          ; make it the current filename
_LTP2:  mov     al,[edx]
        mov     cl,al                   ; ecx = file name character
        and     ecx,03Fh                ; 6 bit only
        mov     ah,[_LTPTrans+ecx]      ; get converted character
        mov     [ebx],ah
        inc     edx
        inc     ebx
        test    al,128                  ; until bit 7 is high
        jz      _LTP2
        xor     al,al
        mov     [ebx],al                ; make it ASCIIZ
_LTP1:  ret
EndP ExtractFileName

; *****************************************************************************
;
;                            Save Tape Patches
;
;       (1) This one goes in the actual 'Save Data' part, assumes
;           filename has already been setup.
;       (2) Goes after filename has been got, saves filename in buffer.
;
; *****************************************************************************

SaveTapePatch Proc
        mov     eax,EXT_P               ; save as a .P file
        call    CMFileName
        mov     edx,ebx                 ; name in EDX
        xor     ecx,ecx                 ; ecx = length
        mov     cx,[word ptr Memory+4014h]
        sub     cx,4009h
        inc     cx
        mov     eax,offset Memory+4009h ; eax = data
        call    FSaveMem
        mov     [byte ptr Memory+03A7h],255
        mov     [word ptr _PC],03A6h    ; jump to RST08 call afterwards
        ret
STPName db      'Quick.P',0
        align   dword
EndP SaveTapePatch

SaveTapePatch2 Proc
        mov     eax,[dword ptr _HL]     ; ex de,hl immediately before !
        mov     [dword ptr _DE],eax     ; Function expects DE
        call    ExtractFileName
        mov     [word ptr _DE],1h       ; what the original command did.
                                        ; DE WAS the header size, I've
                                        ; made it smaller.
        ret
EndP SaveTapePatch2

.data                                   ; ZX81->ASCII Conversion
_LTPTrans db    '        ','   "$$:?'
        db      '()><=+-*','/;,.0123'
        db      '456789AB','CDEFGHIJ'
        db      'KLMNOPQR','STUVWXYZ'
.data?
FNTemp  db      128 dup (?)             ; buffer for new filename

end Start
