; *****************************************************************************
; *****************************************************************************
;
;       File    : Rom.Asm
;       Author  : Paul Robson (autismuk@aol.com)
;       Purpose : Load and Hack ROMs
;       Created : 12th November 1997
;       Modified: 12th November 1997
;
;       Functions :
;               ROMZX81                 Load and patch the ZX81 ROM
;               ROMZX80                 Load and patch the ZX80 ROM
;               ROMFORTH                Load and patch the Forth ROM
;               ROMAszMic               Load and patch the AszMic ROM
;
; *****************************************************************************
; *****************************************************************************

.386p 
.model flat
include extern.inc
.code

; *****************************************************************************
;
;                     Load, Shadow and Patch ZX81 ROM
;
; *****************************************************************************

ROMZX81 Proc
        mov     ebx,offset Memory       ; Load ROM
        mov     edx,offset _DRSName
        call    FLoad

        mov     ebx,offset Memory       ; shadow ROM
_DRS2:  mov     eax,[ebx]
        mov     [ebx+2000h],eax
        add     ebx,4
        cmp     ebx,offset Memory+2000h
        jne     _DRS2

        cmp     [IsVanillaROM],0        ; check for Vanilla Rom.
        jnz     _R810

;       Patch for fast Display Rendering Code
;       If the first line in this handler 'FastPaint' is
;       Jmp DefaultCode then this causes the real ZX81 code to
;       do the drawing. This is slower because it's a per scanline
;       basis
;
        mov     [byte ptr Memory+0043h],0EDh
        mov     [byte ptr Memory+0044h],0FDh
;
;       Tape Patches
;
        mov     [byte ptr Memory+0343h],0EDh    ; Tape Load
        mov     [byte ptr Memory+0344h],0FCh

        mov     [byte ptr Memory+0316h],0EDh    ; Tape Data Save
        mov     [byte ptr Memory+0317h],0FBh

        mov     [byte ptr Memory+02FCh],0EDh    ; Tape Name Save
        mov     [byte ptr Memory+02FDh],0FAh
        mov     [byte ptr Memory+02FEh],000h

_R810:  ret
EndP ROMZX81

; *****************************************************************************
;
;                     Load, Shadow and Patch ZX80 ROM
;
; *****************************************************************************

ROMZX80 Proc
        mov     ebx,offset Memory       ; Load ROM
        mov     edx,offset _DRSName2
        call    FLoad

        mov     ebx,offset Memory       ; shadow ROM (this one is 4k)
_R802:  mov     eax,[ebx]
        mov     [ebx+1000h],eax
        mov     [ebx+2000h],eax
        mov     [ebx+3000h],eax
        add     ebx,4
        cmp     ebx,offset Memory+1000h
        jne     _R802

        mov     [IsZX80Hardware],1      ; make ZX80 Hardware mandatory
                                        ; Speed patch,same as ZX81

        cmp     [IsVanillaROM],0        ; check for Vanilla Rom.
        jnz     _R803
                                        ; speed patch - same as ZX81 (see above)
        mov     [byte ptr Memory+0043h],0EDh
        mov     [byte ptr Memory+0044h],0FDh
_R803:
        ret
EndP ROMZX80

; *****************************************************************************
;
;                     Load, Shadow and Patch AszMic ROM
;
; *****************************************************************************

ROMAszMic Proc
        mov     ebx,offset Memory       ; Load ROM
        mov     edx,offset _DRSName4
        call    FLoad

        mov     ebx,offset Memory       ; shadow ROM (this one is 4k)
_RAZ2:  mov     eax,[ebx]
        mov     [ebx+1000h],eax
        mov     [ebx+2000h],eax
        mov     [ebx+3000h],eax
        add     ebx,4
        cmp     ebx,offset Memory+1000h
        jne     _RAZ2

        mov     [IsZX80Hardware],1      ; make ZX80 Hardware mandatory
                                        ; Speed patch,same as ZX81

        cmp     [IsVanillaROM],0        ; check for Vanilla Rom.
        jnz     _RAZ3

_RAZ3:
        ret
EndP ROMAszMic

; *****************************************************************************
;
;                     Load, Shadow and Patch Forth ROM
;
; *****************************************************************************

ROMForth Proc
        mov     ebx,offset Memory       ; Load ROM
        mov     edx,offset _DRSName3
        call    FLoad

        mov     ebx,offset Memory       ; shadow ROM (this one is 8k)
_RFR2:  mov     eax,[ebx]
        mov     [ebx+2000h],eax
        add     ebx,4
        cmp     ebx,offset Memory+2000h
        jne     _RFR2

        cmp     [IsVanillaROM],0        ; check for Vanilla Rom.
        jnz     _RFR3

        mov     [IsRealNMI],1           ; use real NMI code
_RFR3:
        ret
EndP ROMForth

_DRSName  db    'Zx81.Rom',0
_DRSName2 db    'Zx80.Rom',0
_DRSName3 db    'Tree4th.Rom',0
_DRSName4 db    'AszMic.Rom',0

        align   dword

end

